
import java.io.*;
import java.lang.reflect.*;

public class ReflectionDemo
{
    public ReflectionDemo()
    {

    }

    public  void showType(PrintStream out, String className) throws ClassNotFoundException
    {
        String flavour;
                
        Class thisClass     =  Class.forName(className);
        if( thisClass.isInterface( ) )
                flavour = "interface";
        else
                 flavour= "class";

        out.println(flavour + " " + className);
        Class parentClass = thisClass.getSuperclass();
        if (parentClass != null)
        {
            out.println(" extends " + parentClass.getName());
        }
        Class[] interfaces = thisClass.getInterfaces();
        for (Class interf : interfaces)
        {
            out.println("  implements " + interf.getName());
        }
    }

    public static void showMethod(PrintStream out, Method meth)
    {
        Class returnType = meth.getReturnType();
        out.print("\t" + returnType.getName() + " " + meth.getName() + "(");
        Class[] paramTypes = meth.getParameterTypes();
        String prefix = "";
        for (int i=0; i<paramTypes.length; ++i)
        {
            out.print(prefix + paramTypes[i].getName());
            prefix = ", ";
        }
        out.println(")");
    }

    public static void showMethods( PrintStream out, Object obj)
              throws NoSuchMethodException,
                     IllegalAccessException,
                     InvocationTargetException 
    {
      Class cls = obj.getClass();
      out.println(cls.getName());
      for (Method meth : cls.getMethods())
      {
        if (meth.getDeclaringClass() == cls)
        {
          showMethod(out, meth);
        }
      }
    }

    public void showField(PrintStream out, Object obj, String fieldName) 
    {
        try
        {
            Class cls    = obj.getClass();
            Field field  = cls.getField(fieldName);
            Object value = field.get(obj);
            out.println(fieldName + ": " + value);
        }
        catch (NoSuchFieldException e)
        {
            System.err.println(e);
        }
        catch (IllegalAccessException e)
        {
            System.err.println(e);
        }
    }

    public void exec( )
    {
       try
       {
            /*System.out.println("-----------------------");
            showType( System.out, "MyPoint" );

            System.out.println("-----------------------");
            showType( System.out, "EmailSender");
            */

            System.out.println("-----------------------");
            MyPoint myPoint         = new MyPoint("center", 3, 3);
            EmailSender emailSender = new EmailSender( );

            showMethods( System.out, myPoint );
            System.out.println("-----------------------");
            showMethods( System.out, emailSender );
            System.out.println("-----------------------");
           
       }
       catch( Exception exp)
       {
       }
    }

    public static void main( String strarrArgs[] )
    {
        ReflectionDemo reflectDemo = new ReflectionDemo( );
        reflectDemo.exec( );

    }
}
